#include <fstream>
#include <iostream>
#include <list>
#include <string>

#include <arc/configcore.h>
#include <arc/ngconfig.h>

Config NGConfig::Read(std::istream& is) {

	Config config;

	std::string line;
	std::list<std::string> conf;

	while (getline(is, line))
		conf.push_back(line);

	// Default section - allows to read very old ~/.ngrc
	std::string section("UNDEFINED");

	// Loop over lines
	for (std::list<std::string>::iterator lineit = conf.begin();
	     lineit != conf.end(); lineit++) {

		std::string line = *lineit;

		// Find line delimiters - remove external whitespaces
		std::string::size_type pos1 = line.find_first_not_of(" \t");
		std::string::size_type pos2 = line.find_last_not_of(" \t");

		// Drop comments and empty lines
		if (pos1 == std::string::npos || line[pos1] == '#')
			continue;

		// Is this a section - otherwise must be a statement
		if (line[pos1] == '[' && line[pos2] == ']') {
			section = line.substr(pos1 + 1, pos2 - pos1 - 1);
			// Do not really store the result until there is content
			continue;
		}

		std::string::size_type sep = line.find('=');

		// Skip statements with missing =
		// Skip statements where the = is the first or last character
		if (sep == std::string::npos || sep == pos1 || sep == pos2)
			continue;

		std::string::size_type pos3 = line.find_first_not_of(" \t", sep + 1);
		std::string::size_type pos4 = line.find_last_not_of(" \t", sep - 1);

		// Get Attributes
		std::string attr = line.substr(pos1, pos4 - pos1 + 1);
		std::string value = line.substr(pos3, pos2 - pos3 + 1);

		if ((attr[0] == '"' && attr[attr.size() - 1] == '"') ||
		    (attr[0] == '\'' && attr[attr.size() - 1] == '\''))
			attr = attr.substr(1, attr.size() - 2);
		if ((value[0] == '"' && value[value.size() - 1] == '"') ||
		    (value[0] == '\'' && value[value.size() - 1] == '\''))
			value = value.substr(1, value.size() - 2);

		// Make the key the full path
		config.AddConfValue(section + '/' + attr, value);
	}

	return config;
}

void NGConfig::Write(Config conf, std::ostream& os) {

	// Dump config file
	std::string prev_section;

	for (Options::iterator key = conf.GetOptions().begin();
	     key != conf.GetOptions().end(); key++) {
		std::string::size_type pos = key->first.rfind('/');
		std::string section = key->first.substr(0, pos);
		std::string element = key->first.substr(pos + 1);

		if (section != prev_section) {
			prev_section = section;
			os << "\n[" << section << "]" << std::endl;
		}

		for (std::list<std::string>::iterator value = key->second.begin();
		     value != key->second.end(); value++) {
			os << element << '=' << *value << std::endl;
		}
	}
}
