package se.umu.jobsubmission.impl.broker; 

import java.net.InetAddress;
import java.net.URI;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.globus.mds.glue.GLUECERPType;

import org.ggf.schemas.jsdl._2005._06.jsdl.JobDefinition_Type;

import se.umu.jobsubmission.JobPreferences;
import se.umu.jobsubmission.impl.datamanager.DataHelper;


public class StageInTimePredictor implements Predictor {

    public static final int DEFAULT_STAGEIN_TIME = 0; 
    public static final long MIN_BANDWIDTH = 1048576;  // 1 Mbit/s. 
    	
    private DataHelper dataManager = new DataHelper(); 
    
    private static Logger logger = Logger.getLogger(StageInTimePredictor.class); 
    
    
    
    public int predictFromFileSizes(Map files, 
    		Object resourceObject) throws Exception {
    	
    	if(files.size() == 0) {
    		logger.debug("No input files specified"); 
    		return 0; 
    	}
    	
    	int sum = 0; 
    	
    	GLUECERPType resource = (GLUECERPType) resourceObject; 
    	String resourceName = resource.getCluster(0).getName(); 
    	String localHostName = InetAddress.getLocalHost().getCanonicalHostName();
    	
    	Set keys = files.keySet(); 
    	Iterator iter = keys.iterator(); 
    	
    	// For each (non-local) file, add transfer time for file 
    	// to a sum. 
    	while(iter.hasNext()) {
    		
    		String fileName = (String) iter.next(); 
    		URI fileURI = new URI(fileName); 
    		String fileHost = fileURI.getHost(); 
    		String fileProtocol = fileURI.getScheme(); 
    		
    		// File is local
    		if("file".equals(fileProtocol) && 
    				resourceName.equals(localHostName)) {
    			logger.debug("File " + fileName + " is local to host "
    					+ localHostName); 
    		}
    		else if (resourceName.equals(fileHost)) {
    			logger.debug("File " + fileName + " is stored on resource " 
    					+ resourceName);     			
    		}
    		else {
    			// Remote file, compute transfer time and add to sum. 
    			logger.debug("File: " + fileName); 
    			logger.debug("Is not stored on: " + resourceName);     
    			Long size = (Long) files.get(fileName);
    			long fileSize = size.longValue(); 
    			float tmp = fileSize / 
    					(float) StageInTimePredictor.MIN_BANDWIDTH; 
    			int transferTime = (int) (8 * tmp); 
    			logger.debug("It will take " + transferTime + " seconds to "
    					+ " transfer the " + fileSize + " byte large file"); 
    			sum += transferTime; 
    		
    		}
    		
    	}
    	logger.debug("Sum is: " + sum); 
    	sum = (59 + sum) / 60; // Some sort of roof function :-)
    	
    	logger.debug("Sum in minutes: " + sum);     	
    	
    	return sum; 
    }
   
    
    /**
     * Predicts time required to stage input files
     * to a resource. This is only used 
     * if the user does not specify a stage-in time
     * in the job preferences. 
     * 
     * Prediction is based on file sizes and available bandwidth. 
     * Should this prediction fail, a default time is assumed. 
     */
    public int predict(JobDefinition_Type jobDescription, 
    		Object resource, 
			JobPreferences preferences, 
			Object context) { // throws Exception {
    	
    	// predict from file sizes. 
    	// stage in times... 
    	try {
    		Map inputFiles = (Map) context; 
    		int time = predictFromFileSizes(inputFiles, resource); 
    		logger.debug("Stage in time is: " + time); 
    		return time; 
    	} catch (Exception e) {
    		logger.warn("Could not obtain sizes of input files"); 
    	}
    	
    	// Should this also fail, use default stage in time. 
    	logger.warn("Using default stage in time: " + 
    			StageInTimePredictor.DEFAULT_STAGEIN_TIME); 
    	return StageInTimePredictor.DEFAULT_STAGEIN_TIME; 
 
    }

}




