/*
 * Created on Sep 27, 2005
 *
 *
 */
package se.umu.jobsubmission.impl.dispatcher;

import java.io.File;
import java.net.URI;
import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.Calendar;

import javax.xml.rpc.Stub;

import org.globus.wsrf.container.ServiceContainer; 

import org.apache.axis.components.uuid.UUIDGenFactory;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.AttributedURI;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.types.PositiveInteger;
import org.apache.log4j.Logger;
import org.globus.delegation.DelegationConstants;
import org.globus.delegation.DelegationUtil;
import org.globus.exec.generated.CreateManagedJobInputType;
import org.globus.exec.generated.CreateManagedJobOutputType;
import org.globus.exec.generated.JobDescriptionType;
import org.globus.exec.generated.ManagedJobFactoryPortType;
import org.globus.exec.generated.ManagedJobPortType;
import org.globus.exec.utils.ManagedJobFactoryConstants;
import org.globus.exec.utils.client.ManagedJobClientHelper;
import org.globus.exec.utils.client.ManagedJobFactoryClientHelper;
import org.globus.exec.utils.rsl.RSLHelper;
import org.globus.gsi.GlobusCredential;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.mds.glue.GLUECERPType;
import org.globus.rft.generated.TransferRequestType;
import org.globus.rft.generated.TransferType;
import org.globus.rsl.Value;
import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;
import org.ietf.jgss.GSSCredential;

import se.umu.wsag.util.JobSubmissionUtils;

import se.umu.jobsubmission.util.JobDescriptionTranslator;

/**
 * @author tordsson
 *
 * Basic dispatcher for GT4 (WS-GRAM). 
 * The descriptiontranslator needs to be completed. 
 */
public class GT4Dispatcher implements Dispatcher {
	
	private static Logger logger = Logger.getLogger(GT4Dispatcher.class); 
	private JobDescriptionTranslator descriptionTranslator; 
	
	public GT4Dispatcher() {
		descriptionTranslator = (JobDescriptionTranslator) new JSDL2RSL(); 
	}
	
	public void ping(Object resource) throws Exception {
		
		String MJFSURI = ((GLUECERPType)resource).getCluster(0).getUniqueID(); 
		URI uri = new URI(MJFSURI); 
		String containerURI = uri.getHost() + ":" 
			+ ServiceContainer.DEFAULT_SECURE_PORT; 
		//  + uri.getPort(); 
		
        logger.debug("Testing whether container runs at: " + containerURI); 
        if(!(JobSubmissionUtils.ping(containerURI))) {
        	
        	logger.error("No service container found at: " + containerURI); 
			throw new Exception("No service container found at " 
					+ containerURI + " can not submit job"); 
        }
	}
	
	/**
	 * Creates URL to ManagedJobFactoryService, 
	 * sets client security descriptor, 
	 * delegates credential, creates job resource
	 * and hence submits job. 
	 */
	public Object SubmitJob(JobDescriptionType jobDescription, 
			GLUECERPType resourceInfo) throws Exception {
		
		// [protocol://]host[:port][/servicepath]
		
        // Extract hostname part, and build URL to MJFS: 
        // https://127.0.0.1:8443/wsrf/services/ManagedJobFactoryService
        String resourceURI = resourceInfo.getCluster(0).getUniqueID(); 
		
        URI uri = new URI(resourceURI); 
        String resourceHost = uri.getHost(); 

        
        // Cheated a bit just to test...-> it worked
        // resourceHost = "coppi.cs.umu.se"; 
		
		// 3. Resolve factory URL
		URL factoryUrl = ManagedJobFactoryClientHelper.getServiceURL(
				resourceHost).getURL();
		
		logger.debug("Dispatching job to: " + factoryUrl); 
		
        // factoryType = ManagedJobFactoryConstants.FACTORY_TYPE.PBS; 
        EndpointReferenceType factoryEndpoint = ManagedJobFactoryClientHelper.
			getFactoryEndpoint(factoryUrl, 
					ManagedJobFactoryConstants.FACTORY_TYPE.PBS); 
					// DEFAULT_FACTORY_TYPE); 

		ManagedJobFactoryPortType factoryPort = ManagedJobFactoryClientHelper.
				getPort(factoryEndpoint);

		// 4. Set client security properties.
		ClientSecurityDescriptor secDesc = new ClientSecurityDescriptor();
		// secDesc.setGSITransport(Constants.<protection level constant>);
		secDesc.setGSITransport(Constants.ENCRYPTION); // Really guessing here... 
		//secDesc.setAuthz(<Authorization sub-class instance>);
		secDesc.setAuthz(HostAuthorization.getInstance()); 
		
		// GlobusCredential globusCredential = GlobusCredential.getDefaultCredential(); 
		// Get delegated credential, and delegate it further. 
		
		GlobusCredential globusCredential = JobSubmissionUtils.getDelegatedCredential(); 
		
		GSSCredential credential = new GlobusGSSCredentialImpl(globusCredential, 
				GSSCredential.INITIATE_AND_ACCEPT); 
		if(credential != null) {
			secDesc.setGSSCredential(credential); 
		}
		
		((Stub) factoryPort)._setProperty(Constants.CLIENT_DESCRIPTOR, secDesc);
		
        // Delegate Credentials
		// -> Not needed until we do file staging. 
    	logger.debug("Delegating credentials..."); 
    	
    	// Make this more dynamic... 
		String URL = "https://" + resourceHost + ":8443"
				+ DelegationConstants.SERVICE_BASE_PATH  
				// "wsrf/services/" 
		        + DelegationConstants.FACTORY_PATH; 
				// + "DelegationFactoryService"; 		
		
		logger.debug("Delegating credential to: " + URL); 
		
	   	EndpointReferenceType DelegationServiceEPR = new EndpointReferenceType(); 
    	DelegationServiceEPR.setAddress(new Address(URL)); 
    	X509Certificate[] certs = DelegationUtil.getCertificateChainRP(DelegationServiceEPR,
    			secDesc);    	
    	
  		EndpointReferenceType delegationEpr = DelegationUtil.delegate(
				URL, 
				globusCredential, 
				certs[0], 
				true, 
				secDesc); 
    	
		jobDescription.setStagingCredentialEndpoint(delegationEpr);
		jobDescription.setJobCredentialEndpoint(delegationEpr);
		
		TransferRequestType stageOut = jobDescription.getFileStageOut();
		if(stageOut != null) {
			logger.debug("Setting Delegated credential for job stage out"); 
			stageOut.setTransferCredentialEndpoint(delegationEpr);
		}
		else {
			logger.debug("Job stage out does not require delegated credentials"); 
		}
	
        TransferRequestType stageIn = jobDescription.getFileStageIn(); 
        if(stageIn != null) {
        	logger.debug("Setting delegated credential for job stage in"); 
        	stageIn.setTransferCredentialEndpoint(delegationEpr); 
        }
        else {
        	logger.debug("Job stage in does not require delegated credentials"); 
        }
		
        // 7. Creating the job resource.
		CreateManagedJobInputType jobInput = new CreateManagedJobInputType();
		jobInput.setJobID(new AttributedURI("uuid:" + UUIDGenFactory.getUUIDGen()));
		
		// Set job (job resource) termination time
		// TODO: This must be set to a value at least as high
		// as the job run-time, probably much higher due to
		// batch queue waiting. 
		Calendar cal = Calendar.getInstance(); 
		cal.add(Calendar.MINUTE, 30); 		
		jobInput.setInitialTerminationTime(cal);
		jobInput.setJob(jobDescription);
		
		logger.debug("Creating job resource..."); 
		CreateManagedJobOutputType createResponse = factoryPort.
			createManagedJob(jobInput);
		EndpointReferenceType jobEndpoint = createResponse.getManagedJobEndpoint();
		
		// 8. Creating the job service stub
		ManagedJobPortType jobPort = ManagedJobClientHelper.getPort(jobEndpoint);
	
		// This needs proper serialization...
		logger.debug("Job submitted with id: " + jobEndpoint.toString()); 
		
		return jobEndpoint.toString(); 
		
	}
	
	/**
	 * Prepares the rsl for job submission. 
	 * Sets some undefined attributes, add reservation to rsl, 
	 * and modify input files to ensure correct staging if 
	 * the JSS and the client are run on different hosts. 
	 * @param rsl
	 * @param reservation
	 * @param clientURL
	 */
	public void fixRSL(JobDescriptionType rsl, 
			Object reservation, 
			String clientURL) {
		
		// Set cputime and count to default values unless they 
    	// are specified by the user. 
    	if(rsl.getCount() == null) {
    		PositiveInteger count = new PositiveInteger(""
    				+ Dispatcher.DEFAULT_CPU_COUNT); 
    		rsl.setCount(count); 
    	}
    	if(rsl.getMaxCpuTime() == null) {
    		Long cpuTime = new Long((long) Dispatcher.DEFAULT_CPU_TIME); 
    		rsl.setMaxCpuTime(cpuTime); 
    	}
    	
    	// If any input file starts with file:///fileName, replace this 
    	// with gsiftp://clientURL:2811/fileName
    	TransferRequestType stageIn = rsl.getFileStageIn(); 
    	if(stageIn != null) {
    		TransferType transfers[] = stageIn.getTransfer(); 
    		for(int i = 0; i < transfers.length; i++) {
    			String source = transfers[i].getSourceUrl(); 
    			try {
    				URI uri = new URI(source); 
    				if("file".equals(uri.getScheme())) {
    					logger.debug("Changing input file URL");
    					String path = uri.getSchemeSpecificPart(); 
    					File f = new File(path); 
    			    	String newURL = "gsiftp://" 
    			    				+ clientURL
    								+ ":2811" 
    								+ f.getAbsolutePath(); 
    			    	transfers[i].setSourceUrl(newURL); 
    				}
    			} catch(Exception e) {
    				logger.error("Incorrect specified input files URL" 
    						+ source); 
    			}
    		}
    	}
    	
    	// Add advance reservation 
    	if(reservation != null) {
    		
    	}
		
	}
	
    public Object requestJob(Object resource,   
            Object jobDescription, 
            Object reservation, 
            String clientURL) throws Exception {
    	
    	// Throws Exception unless 
    	this.ping(resource); 
  
    	logger.info("Dispatching job using: " + GT4Dispatcher.class); 
    	JobDescriptionType rsl = (JobDescriptionType) 
			this.descriptionTranslator.translate(jobDescription); 
    	
    	logger.debug("Rsl is: "); 
    	logger.debug(RSLHelper.convertToString(rsl)); 
    	
    	fixRSL(rsl, reservation, clientURL); 
    	
    	logger.debug("Final rsl is: "); 
    	logger.debug(RSLHelper.convertToString(rsl)); 
    	
    	Object result = this.SubmitJob(rsl, (GLUECERPType) resource); 
    	
    	return result; 
    	
    }
}
