package se.umu.jobsubmission.impl.informationfinder; 

import java.util.Calendar;
import java.util.List;

import org.apache.axis.message.MessageElement;
import org.apache.log4j.Logger;
import org.globus.mds.glue.BenchmarkType;
import org.globus.mds.glue.ComputingElementType;
import org.globus.mds.glue.GLUECERPType;
import org.globus.mds.glue.RunTimeEnvironmentType;
import org.globus.mds.glue.SubClusterType;
import org.globus.wsrf.utils.AnyHelper;

import se.umu.jobsubmission.Benchmark;
import se.umu.jobsubmission.BenchmarkList;
import se.umu.jobsubmission.util.GLUEUtils;

public class ARCInformationConverter implements InformationConverter {

    private static Logger logger = 
        Logger.getLogger(ARCInformationConverter.class); 


    // Debugging purposes only: 
    public void printGLUE(GLUECERPType GLUECE) {

        SubClusterType cluster = GLUECE.getCluster(0).getSubCluster(0); 
        ComputingElementType ce = GLUECE.getComputingElement(0); 

        logger.debug("<=== GLUE RP ===>");
        logger.debug("Cluster:Name: " + GLUECE.getCluster(0).getName());
        logger.debug("Cluster:UniqueID: " + GLUECE.getCluster(0).getUniqueID());
        logger.debug("ComputingElement:name: " + ce.getName());
        logger.debug("Policy:MaxTotalJobs: " + ce.getPolicy().getMaxTotalJobs());
        logger.debug("Policy:MaxRunningJobs: " + ce.getPolicy().getMaxRunningJobs());
        logger.debug("State:FreeCPUs: " + ce.getState().getFreeCPUs());
        logger.debug("Info:TotalCPUs: " + ce.getInfo().getTotalCPUs());
        logger.debug("MainMemory:RAMSize: " + cluster.getMainMemory().getRAMSize()); 
        logger.debug("FileSystem:AvailableSize: " + cluster.getFileSystem(0).getSize()); 
        RunTimeEnvironmentType REs[] = cluster.getApplicationSoftware().getRunTimeEnvironment(); 
        for(int i = 0; i < REs.length; i++) 
            logger.debug("RunTimeEnv:Name: " + REs[i].getName());
        
        logger.debug("OperatingSystem:Name: " +cluster.getOperatingSystem().getName());
        logger.debug("Architecture:PlatformType: " +cluster.getArchitecture().getPlatformType());
               
        logger.debug("<=== END OF GLUE RP ===>");

    }

    // GT4's implementation of glue-1.1 specifies ClusterType and HostType, 
    // and GLUECERPType to map these together. 
    //
    // An ARCResource is represented as a GLUECERPType
    //
    // the convertInformation method contains information 
    // each mapped nordugrid attributes and what glue attribute
    // it is mapped on. 

    /*
    // Create a complete GLUECERP object, with all subobjects (that we use...)
    public GLUECERPType createGLUECERP() {

        GLUECERPType GLUECERP = new GLUECERPType(); 

        // Cluster part
        
        // Can ARCResource hold >1 queue? In that case, this is too small...
        ClusterType clusters[] = new ClusterType[1]; 
        clusters[0] = new ClusterType(); 

        GLUECERP.setCluster(clusters); 

        SubClusterType subClusters[] = new SubClusterType[1]; 
        subClusters[0] = new SubClusterType(); 
        clusters[0].setSubCluster(subClusters); 

        ArchitectureType architecture = new ArchitectureType(); 
        subClusters[0].setArchitecture(architecture); 
        OperatingSystemType operatingSystem = new OperatingSystemType(); 
        subClusters[0].setOperatingSystem(operatingSystem); 
        ApplicationSoftwareType appSoftware = new ApplicationSoftwareType(); 
        subClusters[0].setApplicationSoftware(appSoftware); 
        FileSystemType fileSystems[] = new FileSystemType[1]; 
        fileSystems[0] = new FileSystemType(); 
        subClusters[0].setFileSystem(fileSystems); 
        MainMemoryType mainMemory = new MainMemoryType(); 
        subClusters[0].setMainMemory(mainMemory); 

    
        // ComputingElement part
        ComputingElementType computingElements[] = new ComputingElementType[1]; 
        computingElements[0] = new ComputingElementType(); 

        GLUECERP.setComputingElement(computingElements); 

        InfoType info = new InfoType(); 
        computingElements[0].setInfo(info); 
        StateType state = new StateType(); 
        computingElements[0].setState(state); 
        PolicyType policy = new PolicyType(); 
        computingElements[0].setPolicy(policy); 

        return GLUECERP; 

    }*/
    
    // public GLUECERPType convertInformation(ARCResource arcInfo) {
    public Object convertInformation(Object nativeFormat) {

        ARCResource arcInfo = (ARCResource) nativeFormat; 

        GLUECERPType GLUECERP = GLUEUtils.createGLUECERP(); 

        // nordugrid-cluster-architecture -> Architecture:PlatForm
        String architecture = arcInfo.getArchitecture(); 
        if(architecture != null) {
            GLUECERP.getCluster(0).getSubCluster(0).getArchitecture(). 
                setPlatformType(architecture); 
        }

        // nordugrid-cluster-opsys -> OperatingSystem:Name
        // TODO: Might want to parse out version from opsys String. Or not...
        String opsys = arcInfo.getOpsys(); 
        if(opsys != null) {
            GLUECERP.getCluster(0).getSubCluster(0).getOperatingSystem().
                setName(opsys); 
        }

        // nordugrid-cluster-runtimeenvironment -> RuntimeEnvironment:Name
        // Each runtime-env in the list is added as the name to a 
        // RunTimeEnvironmentType
        List runtimeEnvironments = arcInfo.getRuntimeenvironment(); 
        if(runtimeEnvironments != null) {

            int noEnvs = runtimeEnvironments.size(); 
            RunTimeEnvironmentType GLUERuntimeEnvs[] = 
                new RunTimeEnvironmentType[noEnvs]; 

            for(int i = 0; i < noEnvs; i++) {
                GLUERuntimeEnvs[i] = new RunTimeEnvironmentType(); 
                String name = (String) runtimeEnvironments.get(i); 
                GLUERuntimeEnvs[i].setName(name); 
            }
            
            GLUECERP.getCluster(0).getSubCluster(0).getApplicationSoftware().
                setRunTimeEnvironment(GLUERuntimeEnvs); 
        }

        // nordugrid-cluster-sessiondir-free -> FileSystem:AvailableSize
        // TODO: May want to map it to StorageDevice:Size instead. 
        int diskSpace = arcInfo.getDiskspace(); 
        if(diskSpace > 0) { // Error handling isn't that good for int values...
            GLUECERP.getCluster(0).getSubCluster(0).getFileSystem(0). 
                setSize((long)diskSpace); 
        }

        // nordugrid-queue-nodememory -> MainMemory:RAMSize 
        // TODO: Use MainMemory:AvailableSize (#free memory) instead? 
        int nodeMemory = arcInfo.getNodememory(); 
        if(nodeMemory > 0) {
            GLUECERP.getCluster(0).getSubCluster(0).getMainMemory().
                setRAMSize(nodeMemory); 
        }

        // nordugrid-cluster-totalcpus -> Info:TotalCPUs
        // TODO: This is for Glue 1.1, in Glue 1.2., 
        // Info:TotalCPUs is deprecated, use Policy:AssignedCPUs
        // or SubCluster:PhysicalCPUs instead (none of these exist
        // in glue 1.1...)
        int totalCpus = arcInfo.getTotalcpus(); 
        if(totalCpus > 0) {
            GLUECERP.getComputingElement(0).getInfo().
					setTotalCPUs("" +totalCpus); 
        }

        // freejobs = max(cpus-jobs, 0) -> State:FreeCPUs
        // TODO: 
        // Use State:FreeJobsSlots instead in glue1.2
        int freejobs = arcInfo.getFreejobs(); 
        if(freejobs >= 0) { // freejobs may be 0
            GLUECERP.getComputingElement(0).getState().setFreeCPUs(freejobs); 
        }
        
        // nordugrid-cluster-totaljobs -> State:TotalJobs 
        int totalJobs = arcInfo.getTotalJobs(); 
        if(totalJobs >= 0) {
        	GLUECERP.getComputingElement(0).getState().setTotalJobs(totalJobs); 
			
        }

        // nordugrid-queue-maxrunning -> Policy:MaxRunningJobs
        int maxrunning = arcInfo.getMaxrunning(); 
        if(maxrunning > 0) {
            GLUECERP.getComputingElement(0).getPolicy().
                    setMaxRunningJobs(maxrunning); 
        }

        // nordugrid-queue-maxqueable + nordugrid-queue-maxrunning
        //      -> Policy:MaxTotalJobs
        int maxqueable = arcInfo.getMaxqueable(); 
        if(maxqueable > 0 || maxrunning > 0) {
            int total = 0; 
            total += maxqueable > 0 ? maxqueable : 0; 
            total += maxrunning > 0 ? maxrunning : 0; 
            GLUECERP.getComputingElement(0).getPolicy(). 
                setMaxTotalJobs(total); 
        }

        // nordugrid-cluster-name -> ClusterType:Name
        String clusterName = arcInfo.getClustername(); 
        if(clusterName != null) {
            GLUECERP.getCluster(0).setName(clusterName); 
        }

        // nordugrid-cluster-contactstring -> ClusterType:UniqueID
        // N.B: this must not be null!
        String jobURL = arcInfo.getJoburl(); 
        if(jobURL != null) {
            GLUECERP.getCluster(0).setUniqueID(jobURL); 
        }

        // nordugrid-queue-name -> ComputingElement:Name
        String queueName = arcInfo.getQueuename(); 
        if(queueName != null) {
            GLUECERP.getComputingElement(0).setName(queueName); 
        }
        
        // nordugrid-cluster-benchmark -> SubCluster:Benchmark:any... 
        List benchmarks = arcInfo.getBenchmarks(); 
        if(benchmarks != null) {
        	
        	BenchmarkType GLUEBenchmarks = new BenchmarkType(); 
        	
        	BenchmarkList glueBenchmarksList = new BenchmarkList(); 
        	Benchmark[] glueBenchmarks = new Benchmark[benchmarks.size()]; 
        	for(int i = 0; i < benchmarks.size(); i++) {
        		
        		String benchmark = (String) benchmarks.get(i);         		
        		
        		logger.debug("Raw benchmark string is: [" + benchmark + "]"); 
        		// Format is SPECINT2000 @ 1300 
        		int pos = benchmark.indexOf('@'); 
        		String name = benchmark.substring(0, pos).trim(); 
        		String value = benchmark.substring(pos + 1).trim(); 
        		float floatValue = Float.parseFloat(value); 
        		
        		Benchmark b = new Benchmark(); 
        		b.setName(name); 
        		b.setResult(floatValue); 
        		glueBenchmarks[i] = b; 
        	}
        	
        	if(benchmarks.size() > 0) {
        		
        		logger.debug("Found some benchmarks!"); 
        		
        		glueBenchmarksList.setBenchmarks(glueBenchmarks); 
        		MessageElement benchmarksElement[] = 
        			AnyHelper.toAnyArray(glueBenchmarksList);
        		GLUEBenchmarks.set_any(benchmarksElement); 
        		
        		GLUECERP.getCluster(0).getSubCluster(0).
					setBenchmark(GLUEBenchmarks); 
        	}
        }
        
        Calendar validTo = arcInfo.getValidTo(); 
        
        CachedResourceInformation info = new CachedResourceInformation(validTo, 
        		GLUECERP); 

        // return GLUECERP; 
        return info; 
    }

}
