package se.umu.jobsubmission.impl.informationfinder; 

import java.util.Date; 
import java.util.LinkedList;
import java.util.List;
import java.util.Calendar; 

import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import org.apache.log4j.Logger; 

import java.text.SimpleDateFormat; 
 
public class ARCResource {
	
	private static Logger logger = Logger.getLogger(ARCResource.class); 

    private String clustername = null;
    private String joburl = null;
    private String queuename  = null;

    private int freejobs = -1;
    private int maxjobs  = -1; // Use maxrunning and maxqueable instead. 
    private int maxrunning = -1; 
    private int maxqueable = -1; 
    private int totalcpus = -1; 
    private int totaljobs = -1; 

    private String architecture = null;
    private String middleware = null;
    private String opsys = null;
    private List runtimeenvironment = new LinkedList();
    
    private List benchmarks = new LinkedList(); 

    private int diskspace = -1;
    private int nodememory = -1;
    
    private Calendar validTo = null; 

    // Dummy constructor just for testing: 
    public ARCResource(String cn, 
            String ju, 
            String qn, 
            int fj, 
            int mj, 
            int mr, 
            int mq, 
            int tc, 
            String arch, 
            String midd, 
            String opsys, 
            List re, 
            int ds, 
            int nm) {
        this.clustername = cn; 
        this.joburl = ju; 
        this.queuename = qn; 
        this.freejobs = fj; 
        this.maxjobs = mj; 
        this.maxrunning = mr; 
        this.maxqueable = mq; 
        this.totalcpus = tc; 
        this.architecture = arch; 
        this.middleware = midd; 
        this.opsys = opsys; 
        this.runtimeenvironment = re; 
        this.diskspace = ds; 
        this.nodememory = nm; 
    }

    public ARCResource(Attributes cluster, Attributes queue, String validTo) {
        parseAttributes(cluster, queue);
        parseValidTo(validTo); 
    }

    private String setValue(String name,Attributes attrs,String value) {
        try {
            String v = (String)(attrs.get(name).get()); 
            if(v != null) value=v;
        } catch(java.lang.Exception err) {
        }
        return value;
    }

    private int setValue(String name,Attributes attrs,int value) {
        try {
            String v=setValue(name,attrs,null);
            Integer n = new Integer(v);
            value=n.intValue();
        } catch(java.lang.Exception err) {
        }
        return value;
    }

    private void setValues(String name,Attributes attrs,List values) {
        try {
            Attribute attr = attrs.get(name);
            for(int i=0;i<attr.size();i++) {
                String v = (String)(attrs.get(name).get(i));
                if(v == null) continue;
                int n = 0;
                for(;n<values.size();n++) {
                    if(v.equals((String)(values.get(n)))) break;
                }
                if(n >= values.size()) {
                    values.add(v);
                }
            }
        } catch(java.lang.Exception err) {
        }
    }
    
    // Set valid to for this information, useful for caching. 
    private void parseValidTo(String validTo) {
    	
    	// Incoming format is
    	// 20050726120730Z 
    	// Which must be converted into a Calendar
    	// See http://www.timetemperature.com/tzus/time_zone_codes_us.shtml
    	// for more info
    	
    	try {
    		SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmssz"); 
    		if(validTo.charAt(validTo.length() -1) == 'Z')
    			validTo = validTo.substring(0, validTo.length() - 2); 
    		validTo += "GMT"; 
    		Date d = sdf.parse(validTo);
    		this.validTo = Calendar.getInstance(); 
    		this.validTo.setTime(d); 
    		// logger.debug("Valid to: " + this.validTo.getTime()); 
    	} catch (Exception e) {
    		logger.debug("Failed to determine valid_to, assuming 1 minute"); 
    		this.validTo = Calendar.getInstance(); 
    		this.validTo.add(Calendar.MINUTE, 1); 
    	}
    	
    	
    }

    private void parseAttributes(Attributes cluster,Attributes queue) { // ,Attributes user) {

        architecture=setValue("nordugrid-cluster-architecture",cluster,architecture);
        middleware=setValue("nordugrid-cluster-middleware",cluster,middleware);
        opsys=setValue("nordugrid-cluster-opsys",cluster,opsys);
        setValues("nordugrid-cluster-runtimeenvironment",cluster,runtimeenvironment);
        setValues("nordugrid-cluster-benchmark", cluster, benchmarks); 
        
        architecture=setValue("nordugrid-queue-architecture",cluster,architecture);
        middleware=setValue("nordugrid-queue-middleware",cluster,middleware);
        opsys=setValue("nordugrid-queue-opsys",cluster,opsys);
        setValues("nordugrid-queue-runtimeenvironment",cluster,runtimeenvironment);
        setValues("nordugrid-queue-benchmark", cluster, benchmarks); 
        
        architecture=setValue("nordugrid-authuser-architecture",cluster,architecture);
        middleware=setValue("nordugrid-authuser-middleware",cluster,middleware);
        opsys=setValue("nordugrid-authuser-opsys",cluster,opsys);
        setValues("nordugrid-authuser-runtimeenvironment",cluster,runtimeenvironment);
        totaljobs = setValue("nordugrid-cluster-totaljobs", cluster, totaljobs); 
        
        // Modified by Johan 
        joburl=setValue("nordugrid-cluster-contactstring",cluster,joburl);
        queuename=setValue("nordugrid-queue-name",queue,queuename);
        clustername = setValue("nordugrid-cluster-name",cluster,clustername);  

        diskspace=setValue("nordugrid-cluster-sessiondir-free",cluster,diskspace);

        // Memory may be set in only cluster, or only queue...
        nodememory=setValue("nordugrid-queue-nodememory",queue,nodememory);
        if(nodememory < 0) 
            nodememory = setValue("nordugrid-cluster-nodememory", cluster, nodememory); 

        int waitjobs = -1;

        int cpus = -1;
        int jobs = -1;
        int used = -1;
        int queued = -1;
        cpus=setValue("nordugrid-cluster-totalcpus", cluster,cpus);
        jobs=setValue("nordugrid-cluster-totaljobs", cluster,jobs);
        used=setValue("nordugrid-cluster-usedcpus",  cluster,used);
        queued=setValue("nordugrid-cluster-queuedjobs",cluster,queued);
        if((queued>=0) && (used>=0) && ((queued+used)>jobs)) 
        	jobs=queued+used;
        if((jobs>=0) && (cpus>=0)) {
            if(jobs > cpus) {
                freejobs=0;
                waitjobs=jobs-cpus;
            } else {
                freejobs=cpus-jobs;
                waitjobs=0;
            }
        }
        totalcpus = cpus; // Added by Johan 

        used = -1;
        int maxused = -1;
        queued = -1;
        int maxqueued = -1;
        used=setValue("nordugrid-queue-running",queue,used);
        maxused=setValue("nordugrid-queue-maxrunning",queue,maxused);

        // Added by Johan 
        maxrunning = maxused; 
        
        queued=setValue("nordugrid-queue-queued", queue,queued);
        maxqueued=setValue("nordugrid-queue-maxqueuable", queue,maxqueued);

        // Added by Johan
        maxqueable = maxqueued; 
        
        if((cpus>=0) && (cpus<maxused)) maxused=cpus;
        cpus=-1;
        cpus=setValue("nordugrid-queue-totalcpus",queue,cpus);
        if((cpus>=0) && (cpus<maxused)) maxused=cpus;
        cpus=setValue("nordugrid-queue-maxuserrun",queue,cpus);

        // Added by Johan, overwrite any value for cluster with value for queue
        if(cpus >=0) 
            totalcpus = cpus; 
        
        if((cpus>=0) && (cpus<maxused)) maxused=cpus;
        if((used>=0) && (queued>=0)) used+=queued;
        if((maxused>=0) && (used>=0) && (maxused>used)) {
            if(((maxused-used) < freejobs) || (freejobs<0)) freejobs=maxused-used;
        }
        if((maxused>=0) && (maxqueued>=0)) {
            if(((maxused+maxqueued) < maxjobs) || (maxjobs<0)) maxjobs=(maxused+maxqueued);
        }
    
    }

    public void printIt () {

        try {
            if(clustername != null) 
                System.out.println("clustername: " + clustername);
            if(joburl!=null) 
                logger.warn("joburl: "+joburl.toString());
            if(queuename != null) 
                System.out.println("queuename: " + queuename);

            logger.warn("freejobs: "+freejobs);
            // logger.warn("maxjobs: "+maxjobs);            
            logger.warn("maxrunning: "+maxrunning);
            logger.warn("maxqueable: "+maxqueable);
            logger.warn("totalcpus: "+totalcpus);
            
            if(architecture!=null) 
                logger.warn("architecture: "+architecture.toString());
            if(middleware!=null) 
                logger.warn("middleware: "+middleware.toString());
            if(opsys!=null) 
                logger.warn("opsys: "+opsys.toString());
            if(runtimeenvironment!=null) 
                logger.warn("runtimeenvironment: "+runtimeenvironment.toString());
            logger.warn("diskspace: "+diskspace);
            logger.warn("nodememory: "+nodememory);
            
            if(benchmarks != null)
            	logger.warn("benchmarks: " + this.benchmarks.toString()); 
        } catch(java.lang.Exception err) {
            logger.warn("Error: "+err.toString());
        }
    }

    
    // For uniqueness comparasions
    public boolean equals(Object other) {
        if(!(other instanceof ARCResource))
            return false; 
        String otherURL = ((ARCResource)other).getJoburl(); 
        if(otherURL == null) 
            return joburl == null; 
        return otherURL.equals(joburl); 
    }

    // Getters
    public String getClustername() {
        return clustername; 
    }
    public String getJoburl() {
        return joburl; 
    }
    public String getQueuename() {
        return queuename;
    }
    public int getFreejobs() {
        return freejobs; 
    }
    public int getMaxjobs() {
        return maxjobs;
    }
    public int getMaxrunning() {
        return maxrunning; 
    }
    public int getMaxqueable() {
        return maxqueable; 
    }
    public int getTotalcpus() {
        return totalcpus; 
    }
    public String getArchitecture() {
        return architecture; 
    }
    public String getMiddleware() {
        return middleware; 
    }
    public String getOpsys() {
        return opsys; 
    }
    public List getRuntimeenvironment() {
        return runtimeenvironment; 
    }
    public int getDiskspace() {
        return diskspace; 
    }
    public int getNodememory() {
        return nodememory; 
    }
    public List getBenchmarks() {
    	return benchmarks; 
    }    
    public int getTotalJobs() {
    	return totaljobs; 
    }
    public Calendar getValidTo() {
    	return validTo; 
    }

}

