package se.umu.jobsubmission.impl.informationfinder; 

import java.util.List; 
import java.util.LinkedList; 

import javax.naming.directory.Attributes;
import javax.naming.directory.SearchResult;

import org.apache.log4j.Logger; 

import org.nordugrid.job.mds.ARCMDSDiscovery;

import se.umu.jobsubmission.impl.informationfinder.ResourceFinder;

public class ARCResourceFinder implements ResourceFinder {

    // Assume name has format ldap://grid.uio.no:2135/
    //
    static Logger logger = Logger.getLogger(ARCResourceFinder.class); 
    
    public List findResources(String indexServerURI, int timeout) 
    	throws Exception {
        try {
        	// .*[Mm]ds-[Vv]o-[Nn]ame.*
        	if(!indexServerURI.matches(".*[Mm]ds-[Vv]o-[Nn]ame.*")) {
        		logger.error("LDAP server URL is non-ARC: " + indexServerURI); 
        		throw new Exception ("LDAP server URL is non-ARC: " 
        				+ indexServerURI); 
        	}

            logger.debug("Query to: " + indexServerURI); 
            ARCMDSDiscovery arcDiscovery = new ARCMDSDiscovery(indexServerURI, 
            		timeout); 
            arcDiscovery.DiscoverAll(); 

            // Only one return value allowed, make a list of lists, 
            // first list is GIISes, second list is clusters. 
            List resources = new LinkedList(); 

            List giises = new LinkedList(); 
            String giis; 
            int noGIISes = arcDiscovery.haveGIISes(); 
            for(int i = 0; i < noGIISes; i++) {
                giis = arcDiscovery.getGIIS(i); 
                giises.add(giis); 
            }

            List clusters = new LinkedList(); 
            String cluster; 
            int noClusters = arcDiscovery.haveClusters(); 
            for(int i = 0; i < noClusters; i++) {
                cluster = arcDiscovery.getCluster(i); 
                clusters.add(cluster); 
            }

            resources.add(giises); 
            resources.add(clusters); 

            logger.debug("Query to: " + indexServerURI + " done. Found " 
                    + giises.size() + " GIISes and " 
                    + clusters.size() + " clusters"); 

            return resources; 
        }
        catch (Exception e) {
            logger.warn("No resources found at " + indexServerURI); 
            throw new Exception("No resources found at: " + indexServerURI, e); 
        }
    }

}
