package se.umu.jobsubmission.impl.informationfinder; 

import java.util.List; 

import org.apache.log4j.Logger;

import se.umu.jobsubmission.util.Worker; 
import se.umu.jobsubmission.impl.informationfinder.ResourceFinder; 

// Worker class for running ResourceFinders...
public class FinderWorker extends Worker {

    private static Logger logger = Logger.getLogger(FinderWorker.class); 

    private String finderClasses[]; 
    private int timeout; 

    public FinderWorker (Object workUnit, 
    		// ResourceFinder finder, 
    		String finderClasses[], 
			int timeout) {
        super(workUnit); 
        //this.finder = finder; 
        this.finderClasses = finderClasses; 
        this.timeout = timeout; 
    }


    public void doWork() {

        String IS_URI = (String) workUnit; 
        
        // Try to query IS, use one finder at the time until
        // one of them succeedes or all have failed. 
        List resourceList = null; 
        boolean done = false; 
        for(int i = 0; i < this.finderClasses.length && !done; i++) {
        	try {
        		ResourceFinder finder = (ResourceFinder) Class.forName(this.
        				finderClasses[i]).newInstance(); 
        		resourceList = finder.findResources(IS_URI, this.timeout); 
        		// if(resourceList != null && resourceList.size() == 2)
        		done = true; 
        	} catch (Exception e) {
        		logger.warn("Could not query " + IS_URI 
        				+ " using: " + this.finderClasses[i]); 
        		resourceList = null; 
        	}
        }
        
        if(!done) {
        	logger.warn("Coult not query " + IS_URI); 
        	return; 
        }

        // Do something special if an error occurs? 
        // --> This only means that we got no result 
        // from a certain index server, error only occur
        // no resources at all are found...

        if(resourceList.size() != 2) {
            logger.error("Incorrect number of lists returned from finder"); 
            return; 
        }

        // Add new Index servers to workList
        List indexServers = (List) resourceList.get(0); 
        if(indexServers == null) 
            logger.warn("No index servers found"); 
        else {
            logger.debug("" + indexServers.size() + " index servers found"); 
            for(int i = 0; i < indexServers.size(); i++) {

                // Ensure that no index server will be queried twice: 
                String newIS_URI = (String) indexServers.get(i); 

                // Would like to lock here
                // completedWork is a sync-set, but (if!(contains) -> add) 
                // is not synchronized... 
                if(!(completedWork.contains(newIS_URI))) {
                    completedWork.add(newIS_URI); 
                    // and unlock here

                    // Create new worker and add it to pool for execution. 
                    try {
                        FinderWorker newWorker = new FinderWorker(newIS_URI, 
                        		this.finderClasses, 
								this.timeout); 

                        newWorker.connectToThreadPool(resultSet, completedWork, 
                                pool, noActiveTasks, cond); 
                        noActiveTasks.increment(); 
                        logger.debug("Added worker with task: " + newIS_URI); 
                        pool.execute(newWorker); 
                    }
                    catch(Exception e) {
                        logger.error("Error in execution of worker of classes"
                        		+ this.finderClasses  + " : " + e.getMessage()); 
                        // We need not do anything here, 
                        // some other invocation might find resources. 
                        // this case only prunes the resource discovery tree. 
                    }
                }
                else 
                    logger.debug("Avoiding duplicated work: " + newIS_URI); 
            }
        }


        // Add all cluster URIs to resultlist 

        // Usage of a Set ensures that elements are unique. 
        List clusters = (List) resourceList.get(1); 
        if(clusters == null) 
            logger.warn("Found no clusters"); 
        else {
            logger.debug("" + clusters.size() + " resources found"); 
            for(int i = 0; i < clusters.size(); i++) {
                logger.debug("Adding " + clusters.get(i) + " to results"); 
                resultSet.add(clusters.get(i)); 
            }
        }

    }
}

