/*
 * Created on Nov 7, 2005
 *
 * TODO Implement it, then write some documentation
 *
 */
package se.umu.jobsubmission.impl.informationfinder;

import java.util.Hashtable;
import java.util.List; 
import java.util.Vector;

import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;

import org.apache.axis.message.MessageElement;
import org.apache.log4j.Logger;
import org.globus.mds.glue.BenchmarkType;
import org.globus.mds.glue.ComputingElementType;
import org.globus.mds.glue.GLUECERPType;
import org.globus.mds.glue.RunTimeEnvironmentType;
import org.globus.mds.glue.SubClusterType;
import org.globus.mds.glue.PolicyType;
import org.globus.wsrf.utils.AnyHelper;

import se.umu.jobsubmission.Benchmark;
import se.umu.jobsubmission.BenchmarkList;
import se.umu.jobsubmission.util.GLUEUtils;

import org.globus.wsrf.encoding.ObjectDeserializer;

/**
 * @author tordsson
 * Retrieves resource information from one LCG GRIS
 * using LDAP. The retrieved information is in GLUE format, 
 * hence no LCGInformationConverter is required to convert 
 * the information. 
 */
public class LCGInformationFetcher implements InformationFetcher {
	
	private static Logger logger = Logger.getLogger(LCGInformationFetcher.class); 
	
	// TODO: Would really need disk space here as well...
	
    // Debugging purposes only: 
    public void printGLUE(GLUECERPType GLUECE) throws Exception {

        SubClusterType cluster = GLUECE.getCluster(0).getSubCluster(0); 
        ComputingElementType ce = GLUECE.getComputingElement(0); 

        logger.debug("\n\n<=== GLUE RP ===>");
        logger.debug("Cluster:Name: " + GLUECE.getCluster(0).getName());
        logger.debug("Cluster:UniqueID: " + GLUECE.getCluster(0).getUniqueID());
        logger.debug("ComputingElement:name: " + ce.getName());
        logger.debug("Policy:MaxTotalJobs: " + ce.getPolicy().getMaxTotalJobs());
        logger.debug("Policy:MaxRunningJobs: " + ce.getPolicy().getMaxRunningJobs());
        logger.debug("State:FreeCPUs: " + ce.getState().getFreeCPUs());
        logger.debug("Info:TotalCPUs: " + ce.getInfo().getTotalCPUs());
        logger.debug("State:TotalJobs: " +  ce.getState().getTotalJobs()); 
        logger.debug("MainMemory:RAMSize: " + cluster.getMainMemory().getRAMSize()); 
        logger.debug("FileSystem:AvailableSize: " + cluster.getFileSystem(0).getSize()); 
        RunTimeEnvironmentType REs[] = cluster.getApplicationSoftware().getRunTimeEnvironment();        
        if(REs != null) {
        	for(int i = 0; i < REs.length; i++) {
        		logger.debug("RunTimeEnv:Name: " + REs[i].getName());
        	}
        }
        
        logger.debug("OperatingSystem:Name: " +cluster.getOperatingSystem().getName());
        logger.debug("Architecture:PlatformType: " +cluster.getArchitecture().getPlatformType());
        
        MessageElement any[] = cluster.getBenchmark().get_any(); 
        if(any != null) {
        	logger.debug("Benchmarks:\n" + any[0]); 
        }
        
        logger.debug("Benchmarks: " + cluster.getBenchmark().get_any()); 
        logger.debug("<=== END OF GLUE RP ===>\n\n");

    }
	
	
	// GlueClusterTop
	public void buildGLUERPCluster(GLUECERPType glueInfo, Attributes cluster) 
	throws Exception {
		
		SubClusterType subCluster = glueInfo.getCluster(0).getSubCluster(0); 
		
		// GlueHostMainMemoryRAMSize:
		BasicAttribute memory = (BasicAttribute) cluster.get("GlueHostMainMemoryRAMSize"); 
		if(memory != null) {
			long memoryLong = Long.parseLong(memory.get(0).toString()); 
			if(memoryLong > 0) {
				long ramSize = subCluster.getMainMemory().getRAMSize(); 
				glueInfo.getCluster(0).getSubCluster(0).getMainMemory().
						setRAMSize(memoryLong); 
			}
		}
		
		// GlueHostOperatingSystemName:
		BasicAttribute osName = (BasicAttribute) cluster.get("GlueHostOperatingSystemName"); 
		if(osName != null) {
			String nameString = osName.get(0).toString(); 
			subCluster.getOperatingSystem().setName(nameString); 
		}
		
		// GlueClusterName: lxn1184.cern.ch
		BasicAttribute name = (BasicAttribute) cluster.get("GlueClusterName"); 
		if(name != null) {
			String nameString = name.get(0).toString(); 
			glueInfo.getCluster(0).setName(nameString); 
		}
		
		
		// GlueHostApplicationSoftwareRunTimeEnvironment:
		BasicAttribute environment = (BasicAttribute) cluster.
				get("GlueHostApplicationSoftwareRunTimeEnvironment"); 
		if(environment != null) {
			int size = environment.size(); 
			RunTimeEnvironmentType envs[] = new RunTimeEnvironmentType[size]; 
			subCluster.getApplicationSoftware().setRunTimeEnvironment(envs); 
			for(int i = 0; i < size; i++) {
				String env = environment.get(i).toString(); 
				envs[i] = new RunTimeEnvironmentType(); 
				envs[i].setName(env); 
			}
		}
		
		/*
		GlueClusterService: lxn1184.cern.ch:2119/jobmanager-lcgpbs-dteam
		BasicAttribute service = (BasicAttribute) cluster.get("GlueClusterService"); 
		if(service != null) {
			String nameString = name.get(0).toString(); 
			logger.info("GlueClusterService: " + nameString); 
			glueInfo.getCluster(0).setName(nameString); 
		}
		*/
		
		// Other attributes: 
		BasicAttribute specFP = (BasicAttribute) cluster.get("GlueHostBenchmarkSF00"); 
		if(specFP != null) {
			int value = Integer.parseInt(specFP.get(0).toString()); 
			if(value > 0) {
				subCluster.getBenchmark().setSF00((float) value); 
			}
		}
		
		BasicAttribute specINT = (BasicAttribute) cluster.get("GlueHostBenchmarkSI00"); 
		if(specINT != null) {
			int value = Integer.parseInt(specINT.get(0).toString()); 
			if(value > 0) {
				subCluster.getBenchmark().setSI00((float) value); 
			}
		}
		
		// GlueHostBenchmarkSI00: 1234
		// GlueHostBenchmarkSF00: 0
		
	} 
	
	// GlueCETop
	public void buildGLUERPCE(GLUECERPType glueInfo, Attributes ce) throws Exception {
		
		ComputingElementType computingElement = glueInfo.getComputingElement(0); 
		
	    // GlueCEInfoTotalCPUs:
		BasicAttribute cpus = (BasicAttribute) ce.get("GlueCEInfoTotalCPUs"); 
		if(cpus != null) {
			int cpuInt = Integer.parseInt(cpus.get(0).toString()); 
			if(cpuInt > 0) {
		     	String cpuString = computingElement.getInfo().getTotalCPUs(); 
		     	glueInfo.getComputingElement(0).getInfo().setTotalCPUs("" + cpuInt); 
		     }
		}
		
		// GlueCEUniqueID: lxn1184.cern.ch:2119/jobmanager-lcgpbs-dteam
		// This attribute is stored as GlueClusterUniqueID
		// for compatability reasons with and (and most of all), with the broker. 
		BasicAttribute id = (BasicAttribute) ce.get("GlueCEUniqueID"); 
		if(id != null) {
			String idString = id.get(0).toString(); 
			glueInfo.getCluster(0).setUniqueID(idString); 
		}
	
		// GlueCEStateFreeCPUs:
		BasicAttribute freecpus = (BasicAttribute) ce.get("GlueCEStateFreeCPUs"); 
		if(freecpus != null) {
			int cpuInt = Integer.parseInt(freecpus.get(0).toString()); 
			if(cpuInt > 0) {
				computingElement.getState().setFreeCPUs(cpuInt); 
			}
		}
		
		// GlueCEStateTotalJobs: 
		BasicAttribute totalJobs = (BasicAttribute) ce.get("GlueCeStateTotalJobs"); 
		if(totalJobs != null) {
			int jobs = Integer.parseInt(totalJobs.get(0).toString()); 
			int oldJobs = computingElement.getState().getTotalJobs(); 
			if(jobs < oldJobs)
				computingElement.getState().setTotalJobs(jobs); 
		}
		
		// GlueCEPolicyMaxTotalJobs, save max of all advertised values. 
		BasicAttribute maxTotalJobs = (BasicAttribute) ce.get("GlueCEPolicyMaxTotalJobs"); 
		if(maxTotalJobs != null) {
			int maxJobs = Integer.parseInt(maxTotalJobs.get(0).toString()); 
			int oldJobs = computingElement.getPolicy().getMaxTotalJobs(); 
			if(maxJobs > oldJobs) {
				computingElement.getPolicy().setMaxTotalJobs(maxJobs); 
			} 
		}
		
		// GlueCEPolicyMaxRunningJobs, save max of all advertised values. 
		BasicAttribute maxRunningJobs = (BasicAttribute) ce.get("GlueCEPolicyMaxRunningJobs"); 
		if(maxRunningJobs != null) {
			int maxJobs = Integer.parseInt(maxRunningJobs.get(0).toString()); 
			int oldJobs = computingElement.getPolicy().getMaxRunningJobs(); 
			if(maxJobs > oldJobs) {
				computingElement.getPolicy().setMaxRunningJobs(maxJobs); 
			}
		}
		
		// GlueCEName 
		BasicAttribute name = (BasicAttribute) ce.get("GlueCEName"); 
		if(name != null) {
			String nameString = name.get(0).toString(); 
			computingElement.setName(nameString); 
		}
		
	}	
	
	/**
	 * Adds the advertised benchmarks (if they exist), 
	 * to the any element of the subcluster->benchmark object. 
	 * @param glueInfo
	 */
	public void fixBenchmarks(GLUECERPType glueInfo) {
		
		BenchmarkType benchmarks = glueInfo.getCluster(0).getSubCluster(0).getBenchmark(); 
		
		// Collect benchmarks 
		float specFP = benchmarks.getSF00(); 
		float specINT = benchmarks.getSI00(); 
		List benchmarkList = new Vector();
		if(specINT > 0) {
			Benchmark si = new Benchmark(); 
			si.setResult(specINT); 
			si.setName("SPECINT2000"); // TODO: Check this name for compability
			benchmarkList.add(si); 
		}
		if(specFP > 0) {
			Benchmark sf = new Benchmark(); 
			sf.setResult(specFP); 
			sf.setName("SPECFP2000"); // TODO: Check name for compability
			benchmarkList.add(sf); 
		}
		
		// Create benchmarkList data type, serialize it and 
		// add it to any element in subcluster->benchmark object. 
		int size = benchmarkList.size(); 
		if(size > 0) {
			logger.info("Fixing benchmarks..."); 
			Benchmark[] benchmarkArray = new Benchmark[size]; 
			for(int i = 0; i < size; i++) {
				benchmarkArray[i] = (Benchmark) benchmarkList.get(i); 
			}
			BenchmarkList glueBenchmarkList = new BenchmarkList(); 
			glueBenchmarkList.setBenchmarks(benchmarkArray); 
			MessageElement benchmarkElement[] = 
    			AnyHelper.toAnyArray(glueBenchmarkList);
			benchmarks.set_any(benchmarkElement); 
		}
		
	}
	
	/**
	 * The url passed to getInformation can either have the format
	 * "ldap://host:2135/mds-vo-name=local,o=grid"
	 * or simply "host"
	 */
	public String translateURL(String URL) {
		
    	if(URL.startsWith("ldap://"))
    		return URL; 
    	else {
    		return "ldap://" + URL + ":2135/mds-vo-name=local,o=grid"; 
    	}
	}
    
	/**
	 * Retrieves information from resourceURL (if that can be done 
	 * in less than timeout milliseconds). The retrieved information
	 * is stored in a GLUECERPType object which is returned. 
	 */
    public Object getInformation(String resourceURL, int timeout) 
    	throws NamingException {
    	
    	DirContext ctx = null; 
    	
        // This test ensures that the address is directed to a ARC ldap server, 
        // not a lcg one. 
        if(resourceURL.matches(".*[Nn]ordugrid-[Cc]luster-[Nn]ame.*")) {
        	logger.info("ARC LDAP server URL specified: "+ resourceURL); 
        	throw new NamingException("ARC LDAP server URL specified: " 
        			+ resourceURL); 
        }
        
        // Add protocol, host and query to URL unless these are already included.
    	String ldapURL = translateURL(resourceURL); 
    	
    	GLUECERPType glueInfo = GLUEUtils.createGLUECERP(); 

    	try {
    	    Hashtable env = new Hashtable();
            env.put(Context.INITIAL_CONTEXT_FACTORY,
                          "com.sun.jndi.ldap.LdapCtxFactory");
            env.put(Context.PROVIDER_URL, ldapURL);
            env.put("com.sun.jndi.ldap.connect.timeout", ""+timeout);
            ctx = new InitialDirContext(env);
            SearchControls controls = new SearchControls();
            controls.setCountLimit(0); // No limit, return all entries
            controls.setTimeLimit(timeout);
            controls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            
            String filter = "(|(objectclass=GlueCETop)" 
            				+ "(objectclass=GlueClusterTop))"; 
            
            NamingEnumeration results = ctx.search("", filter, controls);
            
            while(results.hasMore()) {
            	
            	SearchResult result = (SearchResult)(results.next());
            	
            	Attributes attributes = result.getAttributes();
            	Attribute objectClass = attributes.get("objectClass");             	
            	String className = (String) objectClass.get(0); 
            	
            	if("GlueCETop".equalsIgnoreCase(className)) {
            		this.buildGLUERPCE(glueInfo, attributes); 
            	}
            	if("GlueClusterTop".equalsIgnoreCase(className)) {
            		this.buildGLUERPCluster(glueInfo, attributes); 
            	}
            	
            }
            
        	// Set maxtotaljobs and maxrunningjobs to default values
        	// unless any queue advertises the attributes. 
        	PolicyType policy = glueInfo.getComputingElement(0).getPolicy(); 
        	int maxTotal = policy.getMaxTotalJobs(); 
        	if(maxTotal == 0) {
        		policy.setMaxTotalJobs(GLUEUtils.DEFAULT_MAX_TOTAL_JOBS); 
        	}
        	int maxRunning = policy.getMaxRunningJobs(); 
        	if(maxRunning == 0) {
        		policy.setMaxRunningJobs(GLUEUtils.DEFAULT_MAX_RUNNING_JOBS); 
        	}
        	
        	// If any benchmarks are set, add these to 
        	// the any element in cluster->subcluster->benchmark 
        	// for compability reasons
        	fixBenchmarks(glueInfo); 
            
            this.printGLUE(glueInfo); 
                    
            return glueInfo; 
            
    	} catch (Exception e) {
    		logger.info("Could not query: " + ldapURL);
    		// Really ugly
    		throw new NamingException(e.getStackTrace().toString()); 
    	}
    	finally {
    		if(ctx != null) {
    			try {
    				ctx.close();                   
    			} catch (Exception e) {
    				logger.warn("Error shutting down connection"); 
    			}
    		}
    	}

    }
}
