package se.umu.jobsubmission.util; 

import org.apache.log4j.Logger;

import EDU.oswego.cs.dl.util.concurrent.SyncSet;
import EDU.oswego.cs.dl.util.concurrent.SynchronizedInt;

// For Workers running tasks in a
// se.umu.jobsubmission.util.ThreadPool
public abstract class Worker implements Runnable {

    // Modifier is protected as subclasses need to access attributes
    
    protected static Logger logger = Logger.getLogger(Worker.class); 

    protected SyncSet resultSet; // Set of results
    protected SyncSet completedWork; // To avoid work duplication

    // For synchronization with thread running threadpool. 
    protected SynchronizedInt noActiveTasks; 
    protected Object cond; 

    protected ThreadPool pool; 

    protected Object workUnit; 

    public Worker(Object workUnit) {
        this.workUnit = workUnit; 
    }

    // Connects the Worker with the ThreadPool (for instanciation when 
    // constructor could not be used)
    public void connectToThreadPool(SyncSet resultSet, SyncSet completedWork, 
            ThreadPool pool, SynchronizedInt noActiveTasks, Object cond) {
        this.resultSet = resultSet; 
        this.completedWork = completedWork; 
        this.pool = pool; 
        this.noActiveTasks = noActiveTasks; 
        this.cond = cond; 
    }

    public void run() {
        try {
            logger.debug("-- Starting"); 
            doWork(); 
            logger.debug("-- Stopping"); 
        } catch(Exception e) {
        	logger.error("Error executing worker: " + e.getMessage()); 
        }

        noActiveTasks.decrement(); 
        logger.debug("Signaling condition"); 
        synchronized (cond) {
            try {
                cond.notifyAll(); 
            } catch (IllegalMonitorStateException e) {
                logger.error("Error when signalling Threadpool"); 
            }
        }
    }

    // Redefined in subClasses (FinderWorker, FetcherWorker, ConverterWorker)
    public abstract void doWork(); 

    public Object getWorkUnit() {
        return workUnit; 
    }

}
