package se.umu.wsag.util; 

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.util.Iterator;
import java.util.Set;

import javax.xml.namespace.QName;

import org.apache.axis.Constants;
import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.apache.axis.client.AxisClient;
import org.apache.axis.encoding.DeserializationContext;
import org.apache.axis.encoding.SerializationContext;
import org.apache.axis.encoding.Serializer;
import org.apache.axis.message.RPCElement;
import org.apache.axis.message.SOAPBodyElement;
import org.apache.axis.message.SOAPEnvelope;
import org.apache.axis.server.AxisServer;
import org.ggf.ws_agreement.ServiceNameSet;
import org.globus.axis.util.Util;
import org.globus.gsi.GlobusCredential;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.gsi.jaas.JaasSubject;
import org.globus.wsrf.client.ServiceURL;
import org.xml.sax.InputSource;

public class JobSubmissionUtils {
	
	private static int PING_TIMEOUT = 2000; 
	
    static {
        Util.registerTransport(); 
    }
    
    /**
	 * Really ugly way to test if container (or any other service) is running.
	 * Unless a socket can be opened, we assume that the service
	 * is not running.	 
	 * @param resourceHost
	 * @return
	 */
	public static boolean ping(String resourceHost) {
		
		// ServiceURL url = new ServiceURL(resourceUrl);
		String host = null; 
		int port = 0; 
		Socket socket = null; 
		try {
			port = ServiceURL.getDefaultPort();
			InetSocketAddress address = new InetSocketAddress(resourceHost, 
					port);
			socket = new Socket(); 						
			socket.connect(address, JobSubmissionUtils.PING_TIMEOUT); 
			socket.close(); 
		} catch (Exception e) {
			if(socket != null) {
				try {
					socket.close(); 
				} catch (Exception err) {}
			}				
			return false; 
		}
		
		return true; 
	}
	
    
    public static String serialize(Object obj, QName qName)
    	throws Exception {
    	    
 	    String result =""; 
    	       	 
    	StringWriter sw = new StringWriter(); 
    	                	
    	MessageContext msgContext = new MessageContext(new AxisClient());         	        	        	        	     	   
    	SerializationContext serContext = 
    		new SerializationContext(sw, msgContext); 
    	serContext.setPretty(true);
    	serContext.setDoMultiRefs(true);
    	
    	Serializer serializer = ServiceNameSet.getSerializer(Constants.AXIS_SAX,
 			obj.getClass(), qName); 
  	
    	serializer.serialize(qName, null, obj, serContext);
 	            	 
    	sw.flush(); 
    	return sw.toString();     	

    }

    /** Deserializes an inputstream to an object, 
     * @param input The inputStream 
     * @param qName QName of object to deserialize into
     * @param clazz Class of object to deserialize into
     */
    public static Object deserialize(InputStream input, 
    		QName qName, Class clazz) throws Exception {
    
    	DeserializationContext deserContext;  
    	
    	// Embed what we are trying to read in a SOAPEnvelope
    	SOAPEnvelope envelope = new SOAPEnvelope();
    	SOAPBodyElement body = new SOAPBodyElement(input);        	       
    	envelope.addBodyElement(body);
    	
    	// Create input for deserializer
    	StringReader reader = new StringReader(envelope.toString()); 
    	MessageContext mctx = new MessageContext(new AxisServer());
    	ByteArrayInputStream stream = new ByteArrayInputStream(
    			envelope.toString().getBytes());    	 
    	InputSource is = new InputSource(stream);
    	
    	// Deserialize
    	deserContext = new DeserializationContext(is, mctx, Message.RESPONSE);
    	// deserContext.setProcessingRef(true); 
    	deserContext.parse();
    	
    	// Unpack from SOAPEnvelope. 
    	SOAPEnvelope env = deserContext.getEnvelope();     	
    	RPCElement rpcElem = (RPCElement)env.getFirstBody();
    	Object result =  rpcElem.getValueAsType(qName, clazz);
    	
    	return result; 
    
    }  
    
    /**
     * Gets the globus credential used by the service, 
     * if user specified GSISecureConversation with delegation, 
     * the delegated credential will be returned. 
     * @return
     * @throws Exception
     */
    public static GlobusCredential getDelegatedCredential() throws Exception {
    	Set s = JaasSubject.getCurrentSubject().getPrivateCredentials(); 
    	Iterator it = s.iterator(); 
    	GlobusGSSCredentialImpl cred = (GlobusGSSCredentialImpl) s.toArray()[0]; 
    	GlobusCredential globusCreds = cred.getGlobusCredential(); 
    	return globusCreds; 
    }
    
}

 

